@@ -17,11 +17,27 @@
 //! @{
 
 
+
 template<typename T1, typename T2, typename glue_type>
 inline
 Glue<T1,T2,glue_type>::Glue(const T1& in_A, const T2& in_B)
   : A(in_A)
   , B(in_B)
+  , aux_u32(aux_u32)
+  {
+  arma_extra_debug_sigprint();
+  
+  isnt_same_type<typename T1::elem_type, typename T2::elem_type>::check();
+  }
+
+
+
+template<typename T1, typename T2, typename glue_type>
+inline
+Glue<T1,T2,glue_type>::Glue(const T1& in_A, const T2& in_B, const u32 in_aux_u32)
+  : A(in_A)
+  , B(in_B)
+  , aux_u32(in_aux_u32)
   {
   arma_extra_debug_sigprint();
   
@@ -38,4 +54,5 @@ Glue<T1,T2,glue_type>::~Glue()
   }
 
 
+
 //! @}
