@@ -17,14 +17,17 @@
 //! @{
 
 
-//! 'negate matrix' operation
+
+//! negate operation
 class op_neg
   {
   public:
   
-  template<typename T1>
-  inline static void apply(Mat<typename T1::elem_type>& out, const Op<T1,op_neg>& in);
+  template<typename T1> inline static void apply( Mat<typename T1::elem_type>& out, const     Op<T1,op_neg>& in);
+  template<typename T1> inline static void apply(Cube<typename T1::elem_type>& out, const OpCube<T1,op_neg>& in);
   
   };
 
+
+
 //! @}
