@@ -32,21 +32,11 @@ class op_rand
   
   public:
   
-  template<typename eT>
-  inline static void apply(Mat<eT>& out, const Op<Mat<eT>,op_rand>& in);
+  template<typename T1>
+  inline static void apply(Mat<typename T1::elem_type>& out, const Op<T1,op_rand>& in);
   
   template<typename eT>
-  inline static void apply(Mat<eT>& out, const Op<Col<eT>,op_rand>& in);
-  
-  template<typename eT>
-  inline static void apply(Mat<eT>& out, const Op<Row<eT>,op_rand>& in);
-  
-  template<typename eT>
-  inline static void apply(Col<eT>& out, const Op<Col<eT>,op_rand>& in);
-  
-  template<typename eT>
-  inline static void apply(Row<eT>& out, const Op<Row<eT>,op_rand>& in);
-
+  inline static void apply(Cube<eT>& out, const OpCube<Cube<eT>,op_rand>& in);
   };
 
 
