@@ -32,9 +32,14 @@ class op_trans
   template<typename T1>
   inline static void apply(Mat<typename T1::elem_type>& out, const Op<T1,op_trans>& in);
   
+  
+  #if defined(ARMA_GOOD_COMPILER)
+  
   template<typename T1, typename T2>
   inline static void apply(Mat<typename T1::elem_type>& out, const Op<Glue<T1,T2,glue_plus>,op_trans>& in);
   
+  #endif
+  
   
   // inline static void apply_inplace(mat &out);
   
