@@ -21,19 +21,16 @@ template<typename T1>
 struct get_pod_type
   { typedef T1 pod_type; };
 
-template<>
 template<typename T2>
 struct get_pod_type< std::complex<T2> >
   { typedef T2 pod_type; };
 
 
 
-
 template<typename T>
 struct is_Mat_only
   { static const bool value = false; };
 
-template<>
 template<typename eT>
 struct is_Mat_only< Mat<eT> >
   { static const bool value = true; };
@@ -44,17 +41,14 @@ template<typename T>
 struct is_Mat
   { static const bool value = false; };
 
-template<>
 template<typename eT>
 struct is_Mat< Mat<eT> >
   { static const bool value = true; };
 
-template<>
 template<typename eT>
 struct is_Mat< Row<eT> >
   { static const bool value = true; };
 
-template<>
 template<typename eT>
 struct is_Mat< Col<eT> >
   { static const bool value = true; };
@@ -65,7 +59,6 @@ template<typename T>
 struct is_Row
   { static const bool value = false; };
 
-template<>
 template<typename eT>
 struct is_Row< Row<eT> >
   { static const bool value = true; };
@@ -76,7 +69,6 @@ template<typename T>
 struct is_Col
   { static const bool value = false; };
 
-template<>
 template<typename eT>
 struct is_Col< Col<eT> >
   { static const bool value = true; };
@@ -90,7 +82,6 @@ template<typename T>
 struct is_subview
   { static const bool value = false; };
 
-template<>
 template<typename eT>
 struct is_subview< subview<eT> >
   { static const bool value = true; };
@@ -100,7 +91,6 @@ template<typename T>
 struct is_diagview
   { static const bool value = false; };
 
-template<>
 template<typename eT>
 struct is_diagview< diagview<eT> >
   { static const bool value = true; };
@@ -111,6 +101,30 @@ struct is_diagview< diagview<eT> >
 //
 
 
+
+template<typename T>
+struct is_Cube
+  { static const bool value = false; };
+
+template<typename eT>
+struct is_Cube< Cube<eT> >
+  { static const bool value = true; };
+
+template<typename T>
+struct is_subview_cube
+  { static const bool value = false; };
+
+template<typename eT>
+struct is_subview_cube< subview_cube<eT> >
+  { static const bool value = true; };
+
+
+
+//
+//
+//
+
+
 template<typename T>
 struct is_Op
   { static const bool value = false; };
@@ -119,7 +133,16 @@ template<typename T1, typename op_type>
 struct is_Op< Op<T1,op_type> >
   { static const bool value = true; };
  
+
+template<typename T>
+struct is_OpCube
+  { static const bool value = false; };
  
+template<typename T1, typename op_type>
+struct is_OpCube< OpCube<T1,op_type> >
+  { static const bool value = true; };
+
+
 template<typename T>
 struct is_Glue
   { static const bool value = false; };
@@ -129,6 +152,15 @@ struct is_Glue< Glue<T1,T2,glue_type> >
   { static const bool value = true; };
 
 
+template<typename T>
+struct is_GlueCube
+  { static const bool value = false; };
+ 
+template<typename T1, typename T2, typename glue_type>
+struct is_GlueCube< GlueCube<T1,T2,glue_type> >
+  { static const bool value = true; };
+
+
 template<typename T>
 struct is_glue_times
   { static const bool value = false; };
@@ -157,6 +189,19 @@ struct is_arma_type
 
 
 
+template<typename T1>
+struct is_arma_cube_type
+  {
+  static const bool value
+  =  is_Cube<T1>::value
+  || is_OpCube<T1>::value
+  || is_GlueCube<T1>::value
+  || is_subview_cube<T1>::value
+  ;
+  };
+
+
+
 //
 //
 //
@@ -316,7 +361,7 @@ template<typename T1>
 struct is_complex
   { static const bool value = false; };
 
-template<>
+// template<>
 template<typename eT>
 struct is_complex< std::complex<eT> >
   { static const bool value = true; };
@@ -349,7 +394,7 @@ template<typename T1>
 struct is_supported_complex
   { static const bool value = false; };
 
-template<>
+//template<>
 template<typename eT>
 struct is_supported_complex< std::complex<eT> >
   { static const bool value = ( sizeof(std::complex<eT>) == 2*sizeof(eT) ); };
@@ -551,6 +596,8 @@ template<> struct promote_type<u16, u8> : public promote_type_ok { typedef u16 r
 template<> struct promote_type<s8, u8> : public promote_type_ok { typedef s8 result; };  // s16 ?
 
 
+
+
 //
 // type promotion, mirrored versions
 
@@ -659,7 +706,7 @@ struct upgrade_val
   };
 
 
-template<>
+// template<>
 template<typename T>
 struct upgrade_val<T,T>
   {
@@ -672,7 +719,7 @@ struct upgrade_val<T,T>
 
 //! upgrade a type to allow multiplication with a complex type
 //! e.g. the int in "int * complex<double>" is upgraded to a double
-template<>
+// template<>
 template<typename T, typename T2>
 struct upgrade_val< std::complex<T>, T2 >
   {
@@ -684,7 +731,7 @@ struct upgrade_val< std::complex<T>, T2 >
   };
 
 
-template<>
+// template<>
 template<typename T1, typename T>
 struct upgrade_val< T1, std::complex<T> >
   {
