@@ -29,7 +29,7 @@ class unwrap
 
 
 
-template<>
+//template<>
 template<typename eT>
 class unwrap< Mat<eT> >
   {
@@ -47,7 +47,7 @@ class unwrap< Mat<eT> >
 
 
 
-template<>
+//template<>
 template<typename eT>
 class unwrap< Row<eT> >
   {
@@ -65,7 +65,7 @@ class unwrap< Row<eT> >
 
 
 
-template<>
+//template<>
 template<typename eT>
 class unwrap< Col<eT> >
   {
@@ -118,7 +118,7 @@ class unwrap< Glue<T1, T2, glue_type> >
   };
 
 
-template<>
+//template<>
 template<typename eT>
 class unwrap< subview<eT> >
   {
@@ -135,7 +135,7 @@ class unwrap< subview<eT> >
   };
 
 
-template<>
+//template<>
 template<typename eT>
 class unwrap< diagview<eT> >
   {
@@ -170,7 +170,7 @@ class unwrap_to_elem_access
 
 
 
-template<>
+//template<>
 template<typename eT>
 class unwrap_to_elem_access< Mat<eT> >
   {
@@ -195,7 +195,7 @@ class unwrap_to_elem_access< Mat<eT> >
 
 
 
-template<>
+//template<>
 template<typename eT>
 class unwrap_to_elem_access< Row<eT> >
   {
@@ -219,7 +219,7 @@ class unwrap_to_elem_access< Row<eT> >
 
 
 
-template<>
+//template<>
 template<typename eT>
 class unwrap_to_elem_access< Op< Row<eT>, op_trans> >
   {
@@ -246,7 +246,7 @@ class unwrap_to_elem_access< Op< Row<eT>, op_trans> >
 
 
 
-template<>
+//template<>
 template<typename eT>
 class unwrap_to_elem_access< Col<eT> >
   {
@@ -270,7 +270,7 @@ class unwrap_to_elem_access< Col<eT> >
 
 
 
-template<>
+//template<>
 template<typename eT>
 class unwrap_to_elem_access< Op<Col<eT>, op_trans> >
   {
@@ -342,7 +342,7 @@ class unwrap_to_elem_access< Glue<T1, T2, glue_type> >
   };
 
 
-template<>
+//template<>
 template<typename eT>
 class unwrap_to_elem_access< subview<eT> >
   {
@@ -381,7 +381,7 @@ class unwrap_check
   };
 
 
-template <>
+//template <>
 template<typename eT>
 class unwrap_check< Mat<eT> >
   {
@@ -452,7 +452,7 @@ class unwrap_check< Mat<eT> >
 
 
 
-template <>
+//template <>
 template<typename eT>
 class unwrap_check< Row<eT> >
   {
@@ -521,7 +521,7 @@ class unwrap_check< Row<eT> >
 
 
 
-template <>
+//template <>
 template<typename eT>
 class unwrap_check< Col<eT> >
   {
@@ -680,7 +680,7 @@ class unwrap_check< Glue<T1, T2, glue_type> >
 
 
 
-template<>
+//template<>
 template<typename eT>
 class unwrap_check< subview<eT> >
   {
@@ -698,7 +698,7 @@ class unwrap_check< subview<eT> >
   };
 
 
-template<>
+//template<>
 template<typename eT>
 class unwrap_check< diagview<eT> >
   {
@@ -716,4 +716,335 @@ class unwrap_check< diagview<eT> >
   };
 
 
+//! if the given object in not a matrix, unwrap it into the given 'out' matrix (i.e. do not create another matrix)
+//! and provide a reference to the 'out' matrix.
+//! if the given object is a matrix, set the size of the 'out' matrix to be the same as the given object
+//! and provide a reference to the given object.
+
+template<typename T1>
+class unwrap_write
+  {
+  private:
+  template<typename eT> inline unwrap_write(Mat<eT>&      out, const T1& in);
+  template<typename eT> inline unwrap_write(Row<eT>&      out, const T1& in);
+  template<typename eT> inline unwrap_write(Col<eT>&      out, const T1& in);
+  template<typename eT> inline unwrap_write(subview<eT>&  out, const T1& in);
+  template<typename eT> inline unwrap_write(diagview<eT>& out, const T1& in);
+  };
+
+
+//template <>
+template<typename eT>
+class unwrap_write< Mat<eT> >
+  {
+  public:
+  
+  inline
+  unwrap_write(Mat<eT>& out, const Mat<eT>& A)
+    : M(A)
+    {
+    arma_extra_debug_sigprint();
+    out.copy_size(A);
+    }
+  
+  
+  inline
+  unwrap_write(Row<eT>& out, const Mat<eT>& A)
+    : M(A)
+    {
+    arma_extra_debug_sigprint();
+    out.copy_size(A);
+    }
+  
+  
+  inline
+  unwrap_write(Col<eT>& out, const Mat<eT>& A)
+    : M(A)
+    {
+    arma_extra_debug_sigprint();
+    out.copy_size(A);
+    }
+  
+  
+  inline
+  ~unwrap_write()
+    {
+    arma_extra_debug_sigprint();
+    }
+  
+  
+  const Mat<eT>& M;
+  };
+
+
+
+//template <>
+template<typename eT>
+class unwrap_write< Row<eT> >
+  {
+  public:
+
+  inline
+  unwrap_write(Mat<eT>& out, const Row<eT>& A)
+    : M(A)
+    {
+    arma_extra_debug_sigprint();
+    out.copy_size(A);
+    }
+  
+  
+  inline
+  unwrap_write(Row<eT>& out, const Row<eT>& A)
+    : M(A)
+    {
+    arma_extra_debug_sigprint();
+    out.copy_size(A);
+    }
+  
+  inline
+  unwrap_write(Col<eT>& out, const Row<eT>& A)
+    : M(A)
+    {
+    arma_extra_debug_sigprint();
+    out.copy_size(A);
+    }
+
+
+  inline
+  ~unwrap_write()
+    {
+    }
+  
+  
+  const Row<eT>& M;
+  };
+
+
+
+
+//template <>
+template<typename eT>
+class unwrap_write< Col<eT> >
+  {
+  public:
+
+  inline
+  unwrap_write(Mat<eT>& out, const Col<eT>& A)
+    : M(A)
+    {
+    arma_extra_debug_sigprint();
+    out.copy_size(A);
+    }
+  
+  
+  inline
+  unwrap_write(Row<eT>& out, const Col<eT>& A)
+    : M(A)
+    {
+    arma_extra_debug_sigprint();
+    out.copy_size(A);
+    }
+  
+  
+  inline
+  unwrap_write(Col<eT>& out, const Col<eT>& A)
+    : M(A)
+    {
+    arma_extra_debug_sigprint();
+    out.copy_size(A);
+    }
+
+
+  inline
+  ~unwrap_write()
+    {
+    }
+  
+  
+  const Col<eT>& M;
+  };
+
+
+
+template<typename T1, typename op_type>
+class unwrap_write< Op<T1, op_type> >
+  {
+  public:
+  typedef typename T1::elem_type eT;
+
+  //template<typename eT>
+  inline
+  unwrap_write(Mat<eT>& out, const Op<T1,op_type>& A)
+    : M(out)
+    {
+    arma_extra_debug_sigprint();
+    out = A;
+    }
+  
+  
+  //template<typename eT>
+  inline
+  unwrap_write(Row<eT>& out, const Op<T1,op_type>& A)
+    : M(out)
+    {
+    arma_extra_debug_sigprint();
+    out = A;
+    }
+  
+  //template<typename eT>
+  inline
+  unwrap_write(Col<eT>& out, const Op<T1,op_type>& A)
+    : M(out)
+    {
+    arma_extra_debug_sigprint();
+    out = A;
+    }
+  
+  
+  inline
+  ~unwrap_write()
+    {
+    arma_extra_debug_sigprint();
+    }
+  
+  const Mat<eT>& M;
+  };
+
+
+
+template<typename T1, typename T2, typename glue_type>
+class unwrap_write< Glue<T1, T2, glue_type> >
+  {
+  public:
+  typedef typename T1::elem_type eT;
+  
+  inline
+  unwrap_write(Mat<eT>& out, const Glue<T1, T2, glue_type>& A)
+    : M(out)
+    {
+    arma_extra_debug_sigprint();
+    out = A;
+    }
+  
+  
+  inline
+  unwrap_write(Row<eT>& out, const Glue<T1, T2, glue_type>& A)
+    : M(out)
+    {
+    arma_extra_debug_sigprint();
+    out = A;
+    }
+  
+  
+  inline
+  unwrap_write(Col<eT>& out, const Glue<T1, T2, glue_type>& A)
+    : M(out)
+    {
+    arma_extra_debug_sigprint();
+    out = A;
+    }
+  
+  
+  inline
+  ~unwrap_write()
+    {
+    arma_extra_debug_sigprint();
+    }
+  
+  
+  const Mat<eT>& M;
+  };
+
+
+
+
+//template<>
+template<typename eT>
+class unwrap_write< subview<eT> >
+  {
+  public:
+  
+  inline
+  unwrap_write(Mat<eT>& out, const subview<eT>& A)
+    : M(out)
+    {
+    arma_extra_debug_sigprint();
+    out = A;
+    }
+  
+  
+  inline
+  unwrap_write(Row<eT>& out, const subview<eT>& A)
+    : M(out)
+    {
+    arma_extra_debug_sigprint();
+    out = A;
+    }
+  
+  
+  inline
+  unwrap_write(Col<eT>& out, const subview<eT>& A)
+    : M(out)
+    {
+    arma_extra_debug_sigprint();
+    out = A;
+    }
+  
+  
+  inline
+  ~unwrap_write()
+    {
+    arma_extra_debug_sigprint();
+    }
+  
+  
+  const Mat<eT>& M;
+  };
+
+
+//template<>
+template<typename eT>
+class unwrap_write< diagview<eT> >
+  {
+  public:
+  
+  inline
+  unwrap_write(Mat<eT>& out, const diagview<eT>& A)
+    : M(out)
+    {
+    arma_extra_debug_sigprint();
+    out = A;
+    }
+  
+  
+  inline
+  unwrap_write(Row<eT>& out, const diagview<eT>& A)
+    : M(out)
+    {
+    arma_extra_debug_sigprint();
+    out = A;
+    }
+  
+  
+  inline
+  unwrap_write(Col<eT>& out, const diagview<eT>& A)
+    : M(out)
+    {
+    arma_extra_debug_sigprint();
+    out = A;
+    }
+  
+  
+  inline
+  ~unwrap_write()
+    {
+    arma_extra_debug_sigprint();
+    }
+  
+  
+  const Mat<eT>& M;
+  };
+
+
+
 //! @}
